/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.conf.impl;

import java.sql.*;
import java.util.*;
import com.silverwrist.util.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

class ConferenceOps_mysql extends ConferenceOps
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DBUtilities m_utils;             // reference to utilities object
  private PropertySerializer m_psz;        // reference to property serializer

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  ConferenceOps_mysql(DBConnectionPool pool)
  {
    super(pool);
    m_utils = (DBUtilities)(pool.queryService(DBUtilities.class));
    m_psz = (PropertySerializer)(pool.queryService(PropertySerializer.class));

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private final java.util.Date touchConference(Connection conn, int confid, java.util.Date date) throws SQLException
  {
    PreparedStatement stmt = null;
    try
    { // prepare and execute the update statement
      stmt = conn.prepareStatement("UPDATE conferences SET lastupdate = ? WHERE confid = ?;");
      m_utils.setDateTime(stmt,1,date);
      stmt.setInt(2,confid);
      stmt.executeUpdate();
      return date;

    } // end try
    finally
    { // make sure and close out the statement
      SQLUtils.shutdown(stmt);

    } // end finally

  } // end touchConference

  private final java.util.Date touchConference(Connection conn, int confid) throws SQLException
  {
    return this.touchConference(conn,confid,new java.util.Date());

  } // end touchConference

  /*--------------------------------------------------------------------------------
   * Overrides from class OpsBase
   *--------------------------------------------------------------------------------
   */

  public void dispose()
  {
    m_psz = null;
    m_utils = null;
    super.dispose();

  } // end dispose

  /*--------------------------------------------------------------------------------
   * Abstract implementations from class ConferenceOps
   *--------------------------------------------------------------------------------
   */

  Object getProperty(int confid, PropertyKey key) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    String rc_str = null;
    try
    { // get a connection
      conn = getConnection();

      // look up the property
      stmt = conn.prepareStatement("SELECT prop_value FROM conf_props WHERE confid = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setInt(1,confid);
      stmt.setInt(2,key.getNamespaceID());
      stmt.setString(3,key.getName());
      rs = stmt.executeQuery();
      if (!(rs.next()))
	return null;  // property not found

      rc_str = rs.getString(1);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(rc_str);
    if (rc!=null)
      return rc;

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(ConferenceOps_mysql.class,"ConferenceMessages",
						 "property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end getProperty

  DateObjectPair setProperty(int confid, PropertyKey key, Object value) throws DatabaseException
  {
    String serialized_value = m_psz.serializeProperty(value);
    if (serialized_value==null)
    { // serialization exception - throw it
      DatabaseException de = new DatabaseException(ConferenceOps_mysql.class,"ConferenceMessages",
						   "property.serialize");
      de.setParameter(0,key.getName());
      throw de;

    } // end if

    String old_value = null;
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    java.util.Date update = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the table
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES conf_props WRITE, conferences WRITE;");

      // look to see if the property value is already there
      stmt = conn.prepareStatement("SELECT prop_value FROM conf_props WHERE confid = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setInt(1,confid);
      stmt.setInt(2,key.getNamespaceID());
      stmt.setString(3,key.getName());
      rs = stmt.executeQuery();
      if (rs.next())
	old_value = rs.getString(1);
      SQLUtils.shutdown(rs);
      rs = null;
      SQLUtils.shutdown(stmt);

      if (old_value!=null)
      { // prepare the statement to update the existing record
	stmt = conn.prepareStatement("UPDATE conf_props SET prop_value = ? WHERE confid = ? AND nsid = ? "
				     + "AND prop_name = ?;");
	stmt.setString(1,serialized_value);
	stmt.setInt(2,confid);
	stmt.setInt(3,key.getNamespaceID());
	stmt.setString(4,key.getName());

      } // end if
      else
      { // prepare the statement to insert a new record
	stmt = conn.prepareStatement("INSERT INTO conf_props (confid, nsid, prop_name, prop_value) "
				     + "VALUES (?, ?, ?, ?);");
	stmt.setInt(1,confid);
	stmt.setInt(2,key.getNamespaceID());
	stmt.setString(3,key.getName());
	stmt.setString(4,serialized_value);

      } // end else

      stmt.executeUpdate();  // execute it!

      // Touch the conference entry.
      update = touchConference(conn,confid);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

    if (old_value==null)
      return new DateObjectPair(null,update);  // no previous value

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(old_value);
    if (rc!=null)
      return new DateObjectPair(rc,update);

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(ConferenceOps_mysql.class,"ConferenceMessages",
						 "property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end setProperty

  DateObjectPair removeProperty(int confid, PropertyKey key) throws DatabaseException
  {
    String old_value = null;
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    java.util.Date update = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the table
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES conf_props WRITE, conferences WRITE;");

      // look to see if the property value is already there
      stmt = conn.prepareStatement("SELECT prop_value FROM conf_props WHERE confid = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setInt(1,confid);
      stmt.setInt(2,key.getNamespaceID());
      stmt.setString(3,key.getName());
      rs = stmt.executeQuery();
      if (rs.next())
	old_value = rs.getString(1);
      else
	return null;  // no need to remove anything

      SQLUtils.shutdown(rs);
      rs = null;
      SQLUtils.shutdown(stmt);

      // delete the database row
      stmt = conn.prepareStatement("DELETE FROM conf_props WHERE confid = ? AND nsid = ? AND prop_name = ?;");
      stmt.setInt(1,confid);
      stmt.setInt(2,key.getNamespaceID());
      stmt.setString(3,key.getName());
      stmt.executeUpdate();

      // Touch the conference entry.
      update = touchConference(conn,confid);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(old_value);
    if (rc!=null)
      return new DateObjectPair(rc,update);

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(ConferenceOps_mysql.class,"ConferenceMessages",
						 "property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end removeProperty

  int[] getPropertyNamespaceIDs(int confid) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // execute the query!
      stmt = conn.prepareStatement("SELECT DISTINCT nsid FROM conf_props WHERE confid = ?;");
      stmt.setInt(1,confid);
      rs = stmt.executeQuery();

      // read out a list of the namespace IDs
      ArrayList tmp = new ArrayList();
      while (rs.next())
	tmp.add(new Integer(rs.getInt(1)));

      // create and return the array
      int[] rc = new int[tmp.size()];
      for (int i=0; i<tmp.size(); i++)
	rc[i] = ((Integer)(tmp.get(i))).intValue();
      tmp.clear();
      return rc;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getPropertyNamespaceIDs

  Map getAllProperties(int confid, int namespace) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // execute the query!
      stmt = conn.prepareStatement("SELECT prop_name, prop_value FROM conf_props WHERE confid = ? AND nsid = ?;");
      stmt.setInt(1,confid);
      stmt.setInt(2,namespace);
      rs = stmt.executeQuery();

      // prepare the return value
      HashMap rc = new HashMap();
      while (rs.next())
      { // copy data out, deserializing properties as we go
	String key = rs.getString(1);
	Object value = m_psz.deserializeProperty(rs.getString(2));
	if (value==null)
	{ // deserialization exception - throw it
	  DatabaseException de = new DatabaseException(ConferenceOps_mysql.class,"ConferenceMessages",
						       "property.deserialize");
	  de.setParameter(0,key);
	  throw de;

	} // end if

	rc.put(key,value);

      } // end while

      return rc;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getAllProperties

  java.util.Date setName(int confid, String name) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the tables
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES conferences WRITE;");

      // prepare and execute the update
      stmt = conn.prepareStatement("UPDATE conferences SET name = ?, lastupdate = ? WHERE confid = ?;");
      stmt.setString(1,name);
      java.util.Date rc = new java.util.Date();
      m_utils.setDateTime(stmt,2,rc);
      stmt.setInt(3,confid);
      stmt.executeUpdate();
      return rc;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end setName

  void setLastUpdateDate(int confid, java.util.Date date) throws DatabaseException
  {
    Connection conn = null;
    Statement stmt = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the tables
      stmt = conn.createStatement();
      stmt.executeUpdate("LOCK TABLES conferences WRITE;");

      // do the touch...
      touchConference(conn,confid,date);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end setLastUpdateDate

  Set getAliases(int confid) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // create and execute the statement
      stmt = conn.prepareStatement("SELECT alias FROM conf_alias WHERE confid = ?;");
      stmt.setInt(1,confid);
      rs = stmt.executeQuery();

      // get the return value
      TreeSet rc = new TreeSet();
      while (rs.next())
	rc.add(rs.getString(1));
      return Collections.unmodifiableSet(rc);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getAliases

  String getPrimaryAlias(int confid) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // create and execute the statement
      stmt = conn.prepareStatement("SELECT alias FROM conf_alias WHERE confid = ? AND prime = 1;");
      stmt.setInt(1,confid);
      rs = stmt.executeQuery();
      if (rs.next())
	return rs.getString(1);
      else
	return null;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getPrimaryAlias

  java.util.Date setPrimaryAlias(int confid, String alias) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the tables
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES conf_alias WRITE, conferences WRITE;");

      // see if the given alias is already the primary one
      stmt = conn.prepareStatement("SELECT confid FROM conf_alias WHERE confid = ? AND alias = ? AND prime = 1;");
      stmt.setInt(1,confid);
      stmt.setString(2,alias);
      rs = stmt.executeQuery();
      if (rs.next())
	return null;  // already there
      rs.close();
      rs = null;
      stmt.close();

      // reset all flags to 0
      stmt = conn.prepareStatement("UPDATE conf_alias SET prime = 0 WHERE confid = ?;");
      stmt.setInt(1,confid);
      stmt.executeUpdate();
      stmt.close();

      // update the table
      stmt = conn.prepareStatement("UPDATE conf_alias SET prime = 1 WHERE confid = ? AND alias = ?;");
      stmt.setInt(1,confid);
      stmt.setString(2,alias);
      int foo = stmt.executeUpdate();
      if (foo==0)
      { // need to insert this as a new alias
	stmt.close();
	stmt = conn.prepareStatement("INSERT INTO conf_alias (confid, alias, prime) VALUES (?, ?, 1);");
	stmt.setInt(1,confid);
	stmt.setString(2,alias);
	stmt.executeUpdate();

      } // end if

      return touchConference(conn,confid);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end setPrimaryAlias

  java.util.Date addAlias(int confid, String alias) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the tables
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES conf_alias WRITE, conferences WRITE;");

      // look for the alias
      stmt = conn.prepareStatement("SELECT confid FROM conf_alias WHERE confid = ? AND alias = ?;");
      stmt.setInt(1,confid);
      stmt.setString(2,alias);
      rs = stmt.executeQuery();
      if (rs.next())
	return null;  // already there
      rs.close();
      rs = null;
      stmt.close();

      // update the alias table
      stmt = conn.prepareStatement("INSERT INTO conf_alias (confid, alias, prime) VALUES (?, ?, 0);");
      stmt.setInt(1,confid);
      stmt.setString(2,alias);
      stmt.executeUpdate();

      return touchConference(conn,confid);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end addAlias

  java.util.Date removeAlias(int confid, String alias) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the tables
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES conf_alias WRITE, conferences WRITE;");

      // look for the alias
      stmt = conn.prepareStatement("SELECT prime FROM conf_alias WHERE confid = ? AND alias = ?;");
      stmt.setInt(1,confid);
      stmt.setString(2,alias);
      rs = stmt.executeQuery();
      if (!(rs.next()))
	return null;
      else if (rs.getInt(1)==1)
	throw new DatabaseException(ConferenceOps_mysql.class,"ConferenceMessages","no.remove.prime");
      rs.close();
      rs = null;
      stmt.close();

      // remove the alias
      stmt = conn.prepareStatement("DELETE FROM conf_alias WHERE confid = ? AND alias = ? AND prime = 0;");
      stmt.setInt(1,confid);
      stmt.setString(2,alias);
      int foo = stmt.executeUpdate();
      if (foo==0)
	return null;  // no such alias

      return touchConference(conn,confid);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end removeAlias

} // end class ConferenceOps_mysql
