/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.conf.impl;

import java.security.acl.*;
import java.util.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.security.SecurityReferenceMonitor;
import com.silverwrist.venice.iface.*;
import com.silverwrist.venice.conf.ConfNamespaces;
import com.silverwrist.venice.conf.iface.*;
import com.silverwrist.venice.conf.obj.*;

class LinkedConferenceImpl implements VeniceLinkedConference
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DynamicImplLinkedConference m_dobj;
  private LinkedConferenceOps m_ops;
  private ConferenceImpl m_conf;
  private VeniceCommunity m_comm;
  private SecurityReferenceMonitor m_srm;
  private int m_sequence;
  private boolean m_hide;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  LinkedConferenceImpl(LinkedConferenceOps ops, ConferenceImpl conf, VeniceCommunity comm,
		       SecurityReferenceMonitor srm, Map params)
  {
    m_dobj = new DynamicImplLinkedConference(this);
    m_ops = ops;
    m_conf = conf;
    m_comm = comm;
    m_srm = srm;
    m_sequence = ((Integer)(params.get(ConferenceManagerOps.KEY_SEQUENCE))).intValue();
    m_hide = ((Boolean)(params.get(ConferenceManagerOps.KEY_HIDE))).booleanValue();

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private final void testPermission(DynamoUser caller, String perm_namespace, String perm_name,
				    String fail_message) throws DatabaseException, DynamoSecurityException
  {
    try
    { // perform security tests...
      if (caller.equals(m_srm.getAdminUser()))
	return;  // Administrator can do anything
      if (m_conf.getAcl().testPermission(caller,perm_namespace,perm_name))
	return;  // we have the right permission in the conference ACL
      if (m_comm.getAcl().testPermission(caller,perm_namespace,perm_name))
	return;  // we have the right permission in the community ACL
      if (m_srm.getGlobalAcl().testPermission(caller,perm_namespace,perm_name))
	return;  // or we can be authorized by the global ACL

    } // end try
    catch (AclNotFoundException e)
    { // just fall through and throw the exception
    } // end catch

    // throw the DynamoSecurityException
    DynamoSecurityException d = new DynamoSecurityException(LinkedConferenceImpl.class,"ConferenceMessages",
							    fail_message);
    throw d;

  } // end testPermission

  /*--------------------------------------------------------------------------------
   * Implementations from interface DynamicObject
   *--------------------------------------------------------------------------------
   */

  public Object get(String property_name) throws DynamicObjectException
  {
    return m_dobj.get(property_name);

  } // end get

  public void set(String property_name, Object value) throws DynamicObjectException
  {
    m_dobj.set(property_name,value);

  } // end set

  public Object call(String method_name, Object[] parameters) throws DynamicObjectException
  {
    return m_dobj.call(method_name,parameters);

  } // end call

  public Object call(String method_name, List parameters) throws DynamicObjectException
  {
    return m_dobj.call(method_name,parameters);

  } // end call

  public DynamicClass getDClass()
  {
    return m_dobj.getDClass();

  } // end getDClass
  
  /*--------------------------------------------------------------------------------
   * Implementations from interface NamedObject
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns the name of this object.
   *
   * @return The name of this object.
   */
  public String getName()
  {
    return m_conf.getName();

  } // end getName

  /*--------------------------------------------------------------------------------
   * Implementations from interface ObjectProvider
   *--------------------------------------------------------------------------------
   */

  /**
   * Retrieves an object from this <CODE>ObjectProvider</CODE>.
   *
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be retrieved.
   * @return The object reference specified.
   */
  public Object getObject(String namespace, String name)
  {
    return m_conf.getObject(namespace,name);

  } // end getObject

  /*--------------------------------------------------------------------------------
   * Implementations from interface SecureObjectStore
   *--------------------------------------------------------------------------------
   */

  /**
   * Sets an object into this <CODE>SecureObjectStore</CODE>.
   *
   * @param caller The user performing the operation.
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be set.
   * @param value The object to set into the <CODE>SecureObjectStore</CODE>.
   * @return The previous object that was set into the <CODE>SecureObjectStore</CODE> under this namespace and name, or
   *         <CODE>null</CODE> if there was no such object.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error setting the object value.
   * @exception com.silverwrist.dynamo.except.DynamoSecurityException If the specified user is not permitted to
   *            set this object value into this <CODE>SecureObjectStore</CODE>.
   */
  public Object setObject(DynamoUser caller, String namespace, String name, Object value)
      throws DatabaseException, DynamoSecurityException
  {
    return m_conf.setObject(caller,namespace,name,value);

  } // end setObject

  /**
   * Removes an object from this <CODE>SecureObjectStore</CODE>.
   *
   * @param caller The user performing the operation.
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be removed.
   * @return The previous object that was set into the <CODE>SecureObjectStore</CODE> under this namespace and name, or
   *         <CODE>null</CODE> if there was no such object.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error removing the object value.
   * @exception com.silverwrist.dynamo.except.DynamoSecurityException If the specified user is not permitted to
   *            remove this object value from this <CODE>SecureObjectStore</CODE>.
   */
  public Object removeObject(DynamoUser caller, String namespace, String name)
      throws DatabaseException, DynamoSecurityException
  {
    return m_conf.removeObject(caller,namespace,name);

  } // end removeObject

  /**
   * Returns a collection of all object namespaces that have been set into this <CODE>SecureObjectStore</CODE>.
   *
   * @return A {@link java.util.Collection Collection} containing {@link java.lang.String String} objects specifying
   *         all the object namespaces.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error getting the namespace list.
   */
  public Collection getNamespaces() throws DatabaseException
  {
    return m_conf.getNamespaces();

  } // end getNamespaces

  /**
   * Returns a collection of all object names that have been set into this <CODE>SecureObjectStore</CODE> under
   * a given namespace.
   *
   * @param namespace The namespace to look for names under.
   * @return A {@link java.util.Collection Collection} containing {@link java.lang.String String} objects
   *         specifying all the object names for this namespace.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error getting the object name list.
   */
  public Collection getNamesForNamespace(String namespace) throws DatabaseException
  {
    return m_conf.getNamesForNamespace(namespace);

  } // end getNamesForNamespace

  /*--------------------------------------------------------------------------------
   * Implementations from interface VeniceConference
   *--------------------------------------------------------------------------------
   */

  public int getConfID()
  {
    return m_conf.getConfID();

  } // end getConfID

  public void setName(DynamoUser caller, String name) throws DatabaseException, DynamoSecurityException
  {
    m_conf.setName(caller,name);

  } // end setName

  public java.util.Date getCreatedDate()
  {
    return m_conf.getCreatedDate();

  } // end getCreatedDate

  public java.util.Date getLastUpdateDate()
  {
    return m_conf.getLastUpdateDate();

  } // end getLastUpdateDate

  public void setLastUpdateDate(DynamoUser caller, java.util.Date date)
      throws DatabaseException, DynamoSecurityException
  {
    m_conf.setLastUpdateDate(caller,date);

  } // end setLastUpdateDate

  public int getHostsGID()
  {
    return m_conf.getHostsGID();

  } // end getHostsGID

  public DynamoGroup getHosts() throws DatabaseException
  {
    return m_conf.getHosts();

  } // end getHosts

  public DynamoAcl getAcl() throws DatabaseException, AclNotFoundException
  {
    return m_conf.getAcl();

  } // end getAcl

  public Set getAliases() throws DatabaseException
  {
    return m_conf.getAliases();

  } // end getAliases

  public String getPrimaryAlias() throws DatabaseException
  {
    return m_conf.getPrimaryAlias();

  } // end getPrimaryAlias

  public void setPrimaryAlias(DynamoUser caller, String alias) throws DatabaseException, DynamoSecurityException
  {
    m_conf.setPrimaryAlias(caller,alias);

  } // end setPrimaryAlias

  public void addAlias(DynamoUser caller, String alias) throws DatabaseException, DynamoSecurityException
  {
    m_conf.addAlias(caller,alias);

  } // end addAlias

  public void removeAlias(DynamoUser caller, String alias) throws DatabaseException, DynamoSecurityException
  {
    m_conf.removeAlias(caller,alias);

  } // end removeAlias

  /*--------------------------------------------------------------------------------
   * Implementations from interface VeniceLinkedConference
   *--------------------------------------------------------------------------------
   */

  public VeniceCommunity getCommunity()
  {
    return m_comm;

  } // end getCommunity

  public VeniceConference getConference()
  {
    return m_conf;

  } // end getConference

  public int getSequence()
  {
    return m_sequence;

  } // end getSequence

  public void setSequence(DynamoUser caller, int sequence) throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,ConfNamespaces.PERMISSIONS_NAMESPACE,"conference.sequence","auth.sequence");
    synchronized (this)
    { // reset the sequence
      m_ops.setSequence(m_comm.getCID(),m_conf.getConfID(),sequence);
      m_sequence = sequence;

    } // end synchronized block

  } // end setSequence

  public boolean isHidden()
  {
    return m_hide;

  } // end isHidden

  public void setHidden(DynamoUser caller, boolean flag) throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,ConfNamespaces.PERMISSIONS_NAMESPACE,"hide.show","auth.hide.show");
    synchronized (this)
    { // set the flag in the database, then here
      m_ops.setHidden(m_comm.getCID(),m_conf.getConfID(),flag);
      m_hide = flag;

    } // end synchronized block

  } // end setHidden

  public void unlink(DynamoUser caller) throws DatabaseException, DynamoSecurityException
  {
  }
  
} // end class LinkedConferenceImpl
