/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.obj;

import java.util.*;
import com.silverwrist.util.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;

public class DynamicObjectImpl implements DynamicObject
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DynamicClass m_dclass;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  protected DynamicObjectImpl(DynamicClass dclass)
  {
    m_dclass = dclass;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  protected final CallFailureException callFailed(Throwable cause)
  {
    return new CallFailureException(m_dclass,cause);

  } // end callFailed

  protected final ParameterCountException badCount(String method_name, Object[] parameters, int len)
  {
    return new ParameterCountException(m_dclass,method_name,len,parameters.length);

  } // end badCount

  protected final void verifyParameterLength(String method_name, Object[] parameters, int len)
  {
    if (parameters.length!=len)
      throw badCount(method_name,parameters,len);

  } // end verifyParameterLength

  protected final ParameterTypeException badType(String method_name, int ndx, Class expected, Class actual)
  {
    return new ParameterTypeException(m_dclass,method_name,ndx,expected,actual);

  } // end badType

  protected final void verifyParameterType(String method_name, Object[] parameters, int ndx, Class klass)
  {
    if (parameters[ndx]==null)
      return;
    if (!(klass.isInstance(parameters[ndx])))
      throw badType(method_name,ndx,klass,parameters[ndx].getClass());

  } // end verifyParameterType

  protected final int getIntValue(String method_name, Object[] parameters, int ndx)
  {
    if (parameters[ndx]==null)
      throw badType(method_name,ndx,Number.class,null);
    if (parameters[ndx] instanceof Number)
      return ((Number)(parameters[ndx])).intValue();
    try
    { // parse string equivalent as integer
      return Integer.parseInt(parameters[ndx].toString());

    } // end try
    catch (NumberFormatException e)
    { // this is where the type error goes...
      throw badType(method_name,ndx,Integer.class,parameters[ndx].getClass());

    } // end catch

  } // end getIntValue

  protected final boolean getBooleanValue(String method_name, Object[] parameters, int ndx)
  {
    if (parameters[ndx]==null)
      throw badType(method_name,ndx,Boolean.class,null);
    if (parameters[ndx] instanceof Boolean)
      return ((Boolean)(parameters[ndx])).booleanValue();
    if (parameters[ndx] instanceof Number)
      return (((Number)(parameters[ndx])).intValue()!=0);
    String s = parameters[ndx].toString();
    if (StringUtils.isBooleanTrue(s))
      return true;
    if (StringUtils.isBooleanFalse(s))
      return false;
    throw badType(method_name,ndx,Boolean.class,parameters[ndx].getClass());
    
  } // end getBooleanValue

  /*--------------------------------------------------------------------------------
   * Implementations from interface DynamicObject
   *--------------------------------------------------------------------------------
   */

  public Object get(String property_name) throws DynamicObjectException
  {
    throw new CodeNotFoundException(m_dclass,CodeNotFoundException.GET,property_name);

  } // end get

  public void set(String property_name, Object value) throws DynamicObjectException
  {
    throw new CodeNotFoundException(m_dclass,CodeNotFoundException.SET,property_name);

  } // end set

  public Object call(String method_name, Object[] parameters) throws DynamicObjectException
  {
    throw new CodeNotFoundException(m_dclass,CodeNotFoundException.CALL,method_name);

  } // end call

  public Object call(String method_name, List parameters) throws DynamicObjectException
  {
    Object[] parray = new Object[parameters.size()];
    if (parray.length>0)
      parameters.toArray(parray);
    return call(method_name,parray);

  } // end call

  public DynamicClass getDClass()
  {
    return m_dclass;

  } // end getDClass
  
} // end class DynamicObjectImpl
