/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.xmlrpc;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.w3c.dom.*;
import com.silverwrist.util.xml.*;
import com.silverwrist.dynamo.RequestType;
import com.silverwrist.dynamo.Verb;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

/**
 * Implements the {@link com.silverwrist.dynamo.iface.Request Request} interface for all XML-RPC requests; parses
 * the request body.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
class XmlRpcRequest extends BaseDelegatingServiceProvider implements Request
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private HttpServletRequest m_req;            // associated HTTP request
  private String m_method;                     // XML-RPC method
  private HashMap m_params = new HashMap();    // parameters
  private Locale[] m_locales;                  // locale list
  private MemoryObjectStore m_attrs;           // scratchpad for attributes

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  /**
   * Creates a new XML-RPC request, parsing the request body to get the actual request data.
   *
   * @param req The original <CODE>HttpServletRequest</CODE> passed to the servlet.
   * @param verb The verb associated with this request.
   * @param services A {@link com.silverwrist.dynamo.iface.ServiceProvider ServiceProvider} object controlling
   *                 access to request services.
   * @param prev The previous {@link com.silverwrist.dynamo.iface.Request Request} object.  May be <CODE>null</CODE>.
   * @exception com.silverwrist.dynamo.except.RequestParseException If the request data is grossly wrong (these
   *            errors are reported as HTTP error codes).
   * @exception com.silverwrist.dynamo.xmlrpc.FaultCode If an error occurs in parsing the request (these errors
   *            are reported as XML-RPC faults).
   */
  XmlRpcRequest(HttpServletRequest req, Verb verb, ServiceProvider services, Request prev)
      throws RequestParseException, FaultCode
  {
    super("XmlRpcRequest",services);
    if (!(Verb.POST.equals(verb)))
      throw new RequestParseException("XML-RPC requests must be POST requests");
    String ctype = req.getContentType();
    if (!(ctype.startsWith("text/xml")))
      throw new RequestParseException("XML-RPC requests must have an XML content type");

    XMLLoader loader = XMLLoader.get();
    Document request_doc = null;
    try
    { // parse the XML request
      request_doc = loader.load(req.getInputStream(),false);

    } // end try
    catch (XMLLoadException e)
    { // parse error in XML body...
      throw new SystemFaultCode(SystemFaultCode.PARSE_ERROR,e);

    } // end catch
    catch (IOException e)
    { // I/O error in XML body...
      throw new SystemFaultCode(SystemFaultCode.PARSE_ERROR,e);

    } // end catch

    XmlRpcSerializer serializer = XmlRpcSerializer.get();
    try
    { // load the XML body of the request, get the method name and parameters
      Element method_call = loader.getRootElement(request_doc,"methodCall");
      DOMElementHelper method_call_h = new DOMElementHelper(method_call);
      m_method = loader.getSubElementText(method_call_h,"methodName");
      Element params_elt = method_call_h.getSubElement("params");
      if (params_elt!=null)
      { // get all <param/> elements from under <params/> and process them
	List l = loader.getMatchingSubElements(params_elt,"param");
	int ndx = 0;
	Iterator it = l.iterator();
	while (it.hasNext())
	{ // get the <value/> subelement from each <param/> and store it
	  Element elt = (Element)(it.next());
	  Element value_elt = loader.getSubElement(elt,"value");
	  m_params.put(String.valueOf(ndx++),serializer.deserialize(value_elt));

	} // end while

      } // end if (parameters present)

    } // end try
    catch (XMLLoadException e)
    { // translate XML exceptions into RequestParseExceptions
      throw new SystemFaultCode(SystemFaultCode.INVALID_REQUEST,e);

    } // end catch

    m_req = req;
    m_locales = new Locale[1];
    m_locales[0] = Locale.getDefault();
    m_attrs = new MemoryObjectStore(XmlRpcRequest.class.getName(),prev);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Implementations from interface ObjectProvider
   *--------------------------------------------------------------------------------
   */

  /**
   * Retrieves an object from this <CODE>ObjectProvider</CODE>.
   *
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be retrieved.
   * @return The object reference specified.
   */
  public Object getObject(String namespace, String name)
  {
    return m_attrs.getObject(namespace,name);

  } // end getObject

  /*--------------------------------------------------------------------------------
   * Implementations from interface ObjectStore
   *--------------------------------------------------------------------------------
   */

  /**
   * Sets an object into this <CODE>ObjectStore</CODE>.
   *
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be set.
   * @param value The object to set into the <CODE>ObjectStore</CODE>.
   * @return The previous object that was set into the <CODE>ObjectStore</CODE> under this namespace and name, or
   *         <CODE>null</CODE> if there was no such object.
   */
  public Object setObject(String namespace, String name, Object value)
  {
    return m_attrs.setObject(namespace,name,value);

  } // end setObject

  /**
   * Removes an object from this <CODE>ObjectStore</CODE>.
   *
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be removed.
   * @return The previous object that was set into the <CODE>ObjectStore</CODE> under this namespace and name, or
   *         <CODE>null</CODE> if there was no such object.
   */
  public Object removeObject(String namespace, String name)
  {
    return m_attrs.removeObject(namespace,name);

  } // end removeObject

  /**
   * Returns a collection of all object namespaces that have been set into this <CODE>ObjectStore</CODE>.
   *
   * @return A <CODE>java.util.Collection</CODE> containing <CODE>String</CODE> objects specifying all the
   *         object namespaces.
   */
  public Collection getNamespaces()
  {
    return m_attrs.getNamespaces();

  } // end getNamespaces

  /**
   * Returns a collection of all object names that have been set into this <CODE>ObjectStore</CODE> under a given
   * namespace.
   *
   * @param namespace The namespace to look for names under.
   * @return A <CODE>java.util.Collection</CODE> containing <CODE>String</CODE> objects specifying all the
   *         object names for this namespace.
   */
  public Collection getNamesForNamespace(String namespace)
  {
    return m_attrs.getNamesForNamespace(namespace);

  } // end getNamesForNamespace

  /*--------------------------------------------------------------------------------
   * Implementations from interface Request
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns a value indicating the type of this request.
   *
   * @return The type of this request.
   * @see com.silverwrist.dynamo.RequestType#XMLRPC
   */
  public RequestType getType()
  {
    return RequestType.XMLRPC;

  } // end getType

  /**
   * Returns the DNS name or IP address of the server to which the request was made.
   * 
   * @return The server name to which the request was name.
   */
  public String getServerName()
  {
    return m_req.getServerName();

  } // end getServerName

  /**
   * Returns the number of the port on which this request was made.
   *
   * @return The port number on which the request was made, or -1 if it could not be determined.
   */
  public int getServerPort()
  {
    return m_req.getServerPort();

  } // end getServerPort

  /**
   * Returns the portion of the request URI that indicates the context of the request. The context path always comes
   * first in a request URI. The path starts with a "/" character but does not end with a "/" character. For servlets
   * in the default (root) context, this method returns "". The servlet container does not decode this string.
   *
   * @return The context path of the request.
   */
  public String getContextPath()
  {
    return m_req.getContextPath();

  } // end getContextPath

  /**
   * Returns the part of this request's URL that calls the endpoint. This includes either the endpoint name or a path
   * to the endpoint, but does not include any extra path information or a query string.  For XML-RPC requests,
   * this will usually be the value "/RPC2".
   *
   * @return The request path of this request.
   */
  public String getRequestPath()
  {
    return m_req.getServletPath();

  } // end getRequestPath

  /**
   * Returns any extra path information associated with the URL the client sent when it made this request. The extra
   * path information follows the request path but precedes the query string. This method returns <CODE>null</CODE>
   * if there was no extra path information.
   *
   * @return The "extra path" of this request.  If there is no "extra path," returns <CODE>null</CODE>.
   */
  public String getExtraPath()
  {
    return null;

  } // end getExtraPath

  /**
   * Returns the query string that is contained in the request URL after the path.  For XML-RPC requests, this is
   * the name of the method that was called.
   *
   * @return The method name associated with this request.
   */
  public String getQueryString()
  {
    return m_method;

  } // end getQueryString

  /**
   * Returns a value indicating the action associated with this request.  For XML-RPC requests, this verb is
   * always {@link com.silverwrist.dynamo.Verb#POST POST}.
   *
   * @return The action associated with this request.
   * @see com.silverwrist.dynamo.Verb
   */
  public Verb getVerb()
  {
    return Verb.POST;

  } // end getVerb

  /**
   * Returns a string representation of the IP address of the client that made this request.
   *
   * @return The address of the client making the request.
   */
  public String getSourceAddress()
  {
    return m_req.getRemoteAddr();

  } // end getSourceAddress

  /**
   * Returns a java.util.Map of the parameters of this request. For XML-RPC requests, these parameters are passed
   * as an array associated with the method call.
   * <P>The keys of the map are <CODE>String</CODE> objects; the values of the map are arrays of the type specified
   * by the return value of {@link #getParametersEntryClass() getParametersEntryClass()}.  For XML-RPC, the keys
   * are printable-string equivalents of integers representing the parameters' position.
   *
   * @return The parameter map, as specified above.  This map is read-only.
   */
  public Map getParameters()
  {
    return Collections.unmodifiableMap(m_params);

  } // end getParameters

  /**
   * Returns a class specifying the fundamental type of objects contained in the arrays acting as the values of
   * the map returned by {@link #getParameters() getParameters()}.  For XML-RPC, this class is <CODE>Object</CODE>,
   * indicating that the contained objects may be of any type, as the parser has already figured out their precise
   * type.
   *
   * @return The class of array elements within the parameter map.
   */
  public Class getParametersEntryClass()
  {
    return Object.class;

  } // end getParametersEntryClass

  /**
   * Returns a java.util.Map of the data items of this request.  Data items are lengthy streams of data associated
   * with a request.  XML-RPC requests have no data items associated with them.
   *
   * @return The data item map, as specified above.  This map is read-only.
   */
  public Map getDataItems()
  {
    return Collections.EMPTY_MAP;

  } // end getDataItems

  /**
   * Returns an array of <CODE>Locale</CODE> objects indicating, in decreasing order starting with the preferred
   * locale, the locales that are acceptable to the client based on an <CODE>Accept-Language</CODE> header or similar.
   * method.  If the client request doesn't provide an <CODE>Accept-Language</CODE> header or similar indication,
   * this method returns an array containing one <CODE>Locale</CODE>, the default locale for the server.
   *
   * @return An array of preferred <CODE>Locale</CODE> objects for the client.
   */
  public Locale[] getLocales()
  {
    return m_locales;

  } // end getLocales

} // end class XmlRpcRequest
